/*********************************************************************
 *  SPDX-License-Identifier: MIT                                     *
 *  Copyright (C) 2006-2022 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of qaplay.                                     *
 *********************************************************************/
#include "qaplay.h"

Qaplay::Qaplay(QSystemTrayIcon *parent) : QSystemTrayIcon(parent)
{
	slClArgs = QApplication::arguments();		// get command line args
	sFileInvoked = slClArgs.at(1);				// can be song, dir or m3u

	QFileInfo qfi(sFileInvoked);
	sFileInvoked = qfi.absoluteFilePath();
	if (qfi.exists() == false or qfi.isReadable() == false) {
		QMessageBox::warning(0, tr("qaplay error - file not readable!"),
		                        tr("The file you specified for playing does either not exist or is not readable!\n"
									"%1\n \n"
									"Please check the name or whether you have read permission for the file.\n \n"
		                            "The program will exit after you clicked 'OK'...").arg(sSongCur));
		exit(0);
	}

	if (QSystemTrayIcon::isSystemTrayAvailable() == false) {
		QMessageBox::warning(0, tr("qaplay error - system tray not available!"),
		                        tr("Ooops, there is no system tray available on this monitor/(virtual)screen combination!\n"
		                            "Until I figure out to use the standard monitor/(virtual)screen to show the system tray icon we will bail out here!\n"
		                            "Sorry - trying harder in the future...\n"
		                            "The program will exit after you clicked 'OK'..."));
		exit(0);
	}

	iWShown = 0;								// widget shown: 0=none, 1=AboutHelp/about, 2=AboutHelp/help, 3=SetPlayList, 4=EditPlayList, 5=SongList

//	QIcon::setThemeName("oxygen");
//	QIcon::setFallbackThemeName("Adwaita");
	setIcon(QIcon("/usr/share/icons/hicolor/scalable/apps/qaplay.svg"));
	setToolTip(tr("Right-click and select 'Quit' to stop playing:\n%1").arg(sSongCur));

	actionsDef();                   // define menu actions

	menu = new QMenu;
	menuCreate();                   // create tray menu

	iM3uLoaded = -1;
	playPrep();
}

Qaplay::~Qaplay()
{
}

void Qaplay::quit()
{
	exit(0);
}


void Qaplay::settingsRead()
{
	QSettings settings("qaplay", "qaplay");
	
	settings.beginGroup("m3u");
		sM3u = settings.value("m3uPathName", "").toString();	// last used playlist, full path n name
	settings.endGroup();
}

void Qaplay::settingsWrite()
{
	QSettings settings("qaplay", "qaplay");

	settings.beginGroup("m3u");
		settings.setValue("m3uPathName", sM3u);					// last used playlist, full path n name
	settings.endGroup();
	settings.sync();
}

void Qaplay::actionsDef()
{
	actAbout = new QAction(QIcon("/usr/share/qaplay/icons/scalable/about.svg"), tr("&About qaplay"), this);
	actAbout->setStatusTip(tr("Show about"));
	connect(actAbout, SIGNAL(triggered()), this, SLOT(ahAbout()));

	actHelp = new QAction(QIcon("/usr/share/qaplay/icons/scalable/help.svg"), tr("&Help"), this);
	actHelp->setStatusTip(tr("Show Help") );
	connect(actHelp, SIGNAL(triggered()), this, SLOT(ahHelp()));


	actEditPlayListShow = new QAction(QIcon("/usr/share/qaplay/icons/scalable/edit.svg"), tr("&Edit favorites playlist"), this);
	actEditPlayListShow->setStatusTip(tr("Show favorites playlist to add songs to for editing"));
	connect(actEditPlayListShow, SIGNAL(triggered()), this, SLOT(editPlaylistWdgtShow()));

	actEditPlayListDelete = new QAction(QIcon("/usr/share/qaplay/icons/scalable/edit.svg"), tr("Close edit favorites pl&aylist dialog"), this);
	actEditPlayListDelete->setStatusTip(tr("Close edit favorites playlist window"));
	connect(actEditPlayListDelete, SIGNAL(triggered()), this, SLOT(editPlaylistWdgtDelete()));


	actPlayListAdd = new QAction(QIcon("/usr/share/qaplay/icons/scalable/plAdd.svg"), tr("A&dd current song to favorites playlist"), this);
	actPlayListAdd->setStatusTip(tr("Add this song to current favorites playlist"));
	connect(actPlayListAdd, SIGNAL(triggered()), this, SLOT(plCurrentSongAdd()));

	actPlayListPlay = new QAction(QIcon("/usr/share/qaplay/icons/scalable/plPlay.svg"), tr("P&lay favorites playlist"), this);
	actPlayListPlay->setStatusTip(tr("Play current favorites playlist"));
	connect(actPlayListPlay, SIGNAL(triggered()), this, SLOT(plCurrentPlayList()));


	actSetPlayListShow = new QAction(QIcon("/usr/share/qaplay/icons/scalable/set.svg"), tr("Set favorites playl&ist"), this);
	actSetPlayListShow->setStatusTip(tr("Set favorites playlist to add songs to"));
	connect(actSetPlayListShow, SIGNAL(triggered()), this, SLOT(setPlayListWdgtShow()));

	actSetPlayListDelete = new QAction(QIcon("/usr/share/qaplay/icons/scalable/set.svg"), tr("Close set favorites playlist dialog"), this);
	actSetPlayListDelete->setStatusTip(tr("Close set favorites playlist window"));
	connect(actSetPlayListDelete, SIGNAL(triggered()), this, SLOT(setPlayListWdgtDelete()));


	actSongListShow = new QAction(QIcon("/usr/share/qaplay/icons/scalable/songlist.svg"), tr("Show current song&List"), this);
	actSongListShow->setStatusTip(tr("Show current song list"));
	connect(actSongListShow, SIGNAL(triggered()), this, SLOT(songListWdgtShow()));

	actSongListDelete = new QAction(QIcon("/usr/share/qaplay/icons/scalable/songlist.svg"), tr("Close current songlist window"), this);
	actSongListDelete->setStatusTip(tr("Close current song list window"));
	connect(actSongListDelete, SIGNAL(triggered()), this, SLOT(songListWdgtDelete()));


	actInfo = new QAction(QIcon("/usr/share/qaplay/icons/scalable/info.svg"), tr("File/Song I&nfo"), this);
	actInfo->setStatusTip(tr("Show file/song info"));
	connect(actInfo, SIGNAL(triggered()), this, SLOT(playInfoShow()));

	actNext = new QAction(QIcon("/usr/share/qaplay/icons/scalable/media-next.svg"), tr("&Next song"), this);
	actNext->setStatusTip(tr("Next song"));
	connect(actNext, SIGNAL(triggered()), this, SLOT(playNext()));

	actPause = new QAction(QIcon("/usr/share/qaplay/icons/scalable/media-pause.svg"), tr("Pause/&Resume playing"), this);
	actPause->setStatusTip(tr("Toggle pause/resume playing"));
	connect(actPause, SIGNAL(triggered()), this, SLOT(playPause()));

	actPrev = new QAction(QIcon("/usr/share/qaplay/icons/scalable/media-previous.svg"), tr("&Previous song"), this);
	actPrev->setStatusTip(tr("Previous song"));
	connect(actPrev, SIGNAL(triggered()), this, SLOT(playPrev()));

	actStop = new QAction(QIcon("/usr/share/qaplay/icons/scalable/media-stop.svg"), tr("&Stop playing"), this);
	actStop->setStatusTip(tr("Stop playing"));
	connect(actStop, SIGNAL(triggered()), this, SLOT(playStop()));


	actQuit = new QAction(QIcon::fromTheme("application-exit-symbolic.svg"), tr("&Quit"), this);
	actQuit->setStatusTip(tr("Quit playing"));
	connect(actQuit, SIGNAL(triggered()), this, SLOT(quit()));
}

void Qaplay::menuCreate()
{
	menu->clear();
	menu->addAction(QIcon("/usr/share/icons/hicolor/scalable/apps/qaplay.svg"), tr("qaplay"));
	menu->addSeparator();

	menu->addAction(actAbout);					// show about widget
	menu->addAction(actHelp);					// show help widget
	menu->addSeparator();

//	menu->addAction(QIcon("/usr/share/qaplay/icons/qaplay.png"), tr("current playlist: ") + sPlayListCur);
	menu->addAction(actPlayListPlay);			// play currently set playlist
	if (iWShown == 3) {
		menu->addAction(actSetPlayListDelete);	// delete widget to set current playlist/manage playlists
	} else {
		menu->addAction(actSetPlayListShow);	// show widget to set current playlist/manage playlists
	}
	if (iWShown == 4) {
		menu->addAction(actEditPlayListDelete);		// delete widget with current playlist for editing
	} else {
		menu->addAction(actEditPlayListShow);		// show widget with current playlist for editing
	}
	menu->addAction(actPlayListAdd);			// add song to current playlist
	menu->addSeparator();
	
	menu->addAction(actPrev);					// play previous song
	menu->addAction(actNext);					// play next song
	menu->addAction(actPause);					// pause/resume playing
	menu->addAction(actStop);					// stop playing
	menu->addSeparator();

	if (iWShown == 5) {	
		menu->addAction(actSongListDelete);		// delete current songlist
	} else {
		menu->addAction(actSongListShow);		// show current songlist window
	}

	menu->addAction(actInfo);					// show file/song info
	menu->addSeparator();
	
	menu->addAction(actQuit);					// quit
	setContextMenu(menu);
}

void Qaplay::toolTipUpd()						// update tooltip with song currently played
{
	setToolTip(tr("Right-click and select 'Quit' to stop playing:\n%1").arg(slSongs.at(iSongCurListIdx)));
}


//  **************************
//  *****  media player  *****
//  **************************
void Qaplay::playPrep()							// prepare playing
{
	QFileInfo fi(sFileInvoked);
	if (fi.isDir() == true) {						// called with directory
		iInvokedBy = 1;
	} else if (fi.suffix().toLower() == "m3u") {	// called with m3u list
		iInvokedBy = 3;
	} else {										// called with file/song
		sD = fi.path();
		iInvokedBy = 2;
	}
	playReadDir();
}

void Qaplay::playReadDir()						// get all songs in dir, calling path can be file, dir or *.m3u
{
	slSongs.clear();
	if (iInvokedBy < 3) {						// iInvokedBy: -1=unknown, 1=a directory, 2=a song, 3=m3u list
//cout << "playReadDir: sFileInvoked = " << sFileInvoked.toStdString() << endl;                                               // *****  debug  *****
		QFileInfo fi(sFileInvoked);
		QDir dir(fi.absolutePath());								// song directory, look for songs to play in this dir
			slFilters << "*.mp3" << "*.ogg" << "*.flac" << "*.wav" << "*.wma" << "*.aac" << "*.ts" << "*.mkv" << "*.mp4" << "*.webm";	// more?
			dir.setNameFilters(slFilters);
			dir.setFilter(QDir::Files | QDir::Readable);
			dir.setSorting(QDir::Name | QDir::IgnoreCase);
		slD = dir.entryList();

		if (slD.count() > 0) {							// found at least 1 song with filter settings
			for (int i = 0; i < slD.count(); ++i) {			// create song list
				slSongs.append(dir.absolutePath() + "/" + slD.at(i));
			}
			bD = false;
			if (iInvokedBy == 2) {						// called by song, find it in list, that should be played
				for (int i = 0; i < slSongs.count(); ++i) {
					if (slSongs.at(i) == sFileInvoked) {		// song qaplay was called with is among the list of files returned by dir serach
						bD = true;
						iSongCurListIdx = i;
						break;
					}
				}
				if (bD == false) {						// did not found song qaplay was called with in song list
					slSongs.append(sFileInvoked);			// qaplay was called with different song type we support (and set filters ), let's try anyway, add at end
					iSongCurListIdx = slSongs.count() - 1;
				}
			} else {
				iSongCurListIdx = 0;					// play dir: start with 1st song in dir
			}
		} else {
			slSongs.append(sFileInvoked);				// qaplay was called with different song type we support (and set filters ), let's try anyway (can now be a dir!)
			iSongCurListIdx = 0;							// qaplay was invoked with this only file in list
		}
	} else {											// qaplay was called with a m3u playlist
		slSongs = m3uSongGetList(sFileInvoked);
		if (slSongs.at(1).left(3) == "err") {
			showMessage(tr("qaplay information"),tr("Sorry, could not extract a single song from the m3u?!"),QSystemTrayIcon::Information,10000);
			return;
		}
		iSongCurListIdx = 0;								// set index to 1st song
	}
	
	playCreate();
}

void Qaplay::playCreate()
{
	player = new QMediaPlayer;
	connect(player, SIGNAL(mediaStatusChanged(QMediaPlayer::MediaStatus)), this, SLOT(playStatusChanged(QMediaPlayer::MediaStatus)));
	audioOutput = new QAudioOutput;
	player->setAudioOutput(audioOutput);
	playPlay();
}

void Qaplay::playPlay()
{
//cout << "playPlay: Song = " << slSongs.at(iSongCurListIdx).toStdString() << endl;                                               // *****  debug  *****
	player->setSource(QUrl::fromLocalFile(slSongs.at(iSongCurListIdx)));
//	player->setVolume(50);
	toolTipUpd();
	metaData = player->metaData();
	player->play();
}

void Qaplay::playStatusChanged(QMediaPlayer::MediaStatus state)
{
	switch (state) {
		case QMediaPlayer::EndOfMedia:
		    if (slSongs.count() - 1 > iSongCurListIdx) {
				iSongCurListIdx += 1;
				playPlay();
			} else {							// quit at end of list
				quit();
			}
			break;
		case QMediaPlayer::InvalidMedia:
			showMessage(tr("Invalid Media File!"), tr("Cannot play the file:/n%1").arg(slSongs.at(iSongCurListIdx)),QSystemTrayIcon::Information,10000);
			quit();
			break;
	}
}

void Qaplay::playAt()							// play a different song from the song list - prev, next or any other selected in song list widget
{
	player->stop();
	playPlay();
}

void Qaplay::playPrev()
{
	if (iSongCurListIdx > 0) {
		iSongCurListIdx -= 1;
	} else {
		iSongCurListIdx = slSongs.count() - 1;	// roll over, reached begin, start at end of list, last song again...
	}
	playAt();
}

void Qaplay::playNext()
{
	if (slSongs.count() - 1 > iSongCurListIdx) {
		iSongCurListIdx += 1;
	} else {
		iSongCurListIdx = 0;					// roll over, reached end, start at 0, first song again...
	}
	playAt();
}

void Qaplay::playPause()
{
	if (player->playbackState() == QMediaPlayer::PlayingState) {
//		cout << "playPause() - state playing" << endl;                                               // *****  debug  *****
		player->pause();
	} else if (player->playbackState() == QMediaPlayer::PausedState) {
//		cout << "playPause() - state paused" << endl;                                               // *****  debug  *****
		player->play();
	} else {
//		cout << "playPause() - state else" << endl;                                               // *****  debug  *****
		player->play();								// why would we ever end here?
	}
}

void Qaplay::playStop()
{
	player->stop();
	quit();										// for now we stop, as we cannot re-start without playlist !!!!!!!!!!!!!!!!!!!!!!
}

void Qaplay::playInfoShow()						// show song info
{
	strSiD.sArtist = metaData.metaDataKeyToString(QMediaMetaData::AlbumArtist);
	strSiD.sTitle = metaData.metaDataKeyToString(QMediaMetaData::Title);
	strSiD.sAlbum = metaData.metaDataKeyToString(QMediaMetaData::AlbumTitle);
	strSiD.sYear = metaData.metaDataKeyToString(QMediaMetaData::Date);
	strSiD.sComment = metaData.metaDataKeyToString(QMediaMetaData::Comment);
	strSiD.sTrack = metaData.metaDataKeyToString(QMediaMetaData::TrackNumber);
	strSiD.sGenre = metaData.metaDataKeyToString(QMediaMetaData::Genre);
	iD = metaData.value(QMediaMetaData::AudioBitRate).toInt() / 1000;
	strSiD.sBitRate = QString::number(iD);
//	strSiD.sChannels = metaData.metaDataKeyToString(QMediaMetaData::ChannelCount);
	strSiD.sCodec = metaData.metaDataKeyToString(QMediaMetaData::FileFormat);
	sD = "";
	if (strSiD.sArtist != "") {
		sD = strSiD.sArtist;
	}
	if (strSiD.sTitle != "") {
		sD += " - " + strSiD.sTitle;
	}
	if (strSiD.sAlbum != "") {
		sD += " | " + strSiD.sAlbum;
	}
	if (strSiD.sTrack != "") {
		sD += " - " + strSiD.sTrack;
	}
	if (strSiD.sYear != "") {
		sD += " - " + strSiD.sYear;
	}
	if (strSiD.sGenre != "") {
		sD += " | " + strSiD.sGenre;
	}
	if (strSiD.sComment != "") {
		sD += " | " + strSiD.sComment;
	}
	if (strSiD.sBitRate != "") {
		sD += " | " + strSiD.sBitRate + " kbps | ";
	}
	sD += playInfoFormatTime(player->position()) + "/";
	sD += playInfoFormatTime(player->duration());
	if (strSiD.sChannels != "") {
		sD += " | " + strSiD.sChannels;
	}
	if (strSiD.sCodec != "") {
		if (strSiD.sCodec.contains("MP3")) {
			strSiD.sCodec = "mp3";
		} else if(strSiD.sCodec.contains("OGG")) {
			strSiD.sCodec = "ogg";
		} else if(strSiD.sCodec.contains("FLAC")) {
			strSiD.sCodec = "flac";
		}
		sD += " | " + strSiD.sCodec;
	}
//cout << "showInfo() : " << sD.toStdString() << endl;                                               // *****  debug  *****
	if (sD != "") {
		showMessage(tr("Song Info"),sD,QSystemTrayIcon::Information,8000);
	} else {
		showMessage(tr("Song Info"),tr("Ooops! Sorry, could not get any song information from the file currently playing."),QSystemTrayIcon::Information,8000);
	}
}

QString Qaplay::playInfoFormatTime(int iD2)		// format song time - position n duration, expects milliseconds
{
	iD2 = iD2 / 1000;							// turn milliseconds into seconds
	if (iD2 / 60 < 10) {						// get minutes
		sD2 = "0" + QString::number(iD2 / 60) + ":";
	} else {
		sD2 = QString::number(iD2 / 60) + ":";
	}
	iD2 = iD2 - (iD2 / 60 * 60);
	if (iD2 < 10) {						// get minutes
		sD2 += "0" + QString::number(iD2);
	} else {
		sD2 += QString::number(iD2);
	}
	return sD2;
}


//  ****************************************
//  *****  current playlist functions  *****
//  ****************************************
void Qaplay::plCurrentSetup()					// load internal playlist only when first needed - speed path
{
	if (iM3uLoaded == -1) {							// first call, need to check and load playlist if available - here as not in start speed path
		iM3uLoaded = plCurrentCheckAvail();			// -1=sM3u="", 0=not available, 1=exists n writable (not loaded)
	}

	if (iM3uLoaded < 1) {							// create new empty m3u
		sD = m3uFileCreate("");
		if (sD == "0") {
			showMessage(tr("qaplay error"),tr("Sorry, cannot add a song: could not create a playlist in your home directory!"),QSystemTrayIcon::Information,10000);
			return;
		} else {
			sM3u = sD;
			iM3uLoaded = 1;
			settingsWrite();						// save name of current playlist
		}
	}

	if (iM3uLoaded == 1) {							// load m3u into slM3u - there are currently no checks performed if we succeed !
		plCurrentLoad();
	}
}

int Qaplay::plCurrentCheckAvail()				// check whether a previous used playlist is available
{
	settingsRead();									// get last m3u if any
	if (sM3u != "") {
		return(m3uFileIsWritable(sM3u));			// 0=not available, 1=exists n writable
	} else {
		return 0;									// make compiler happy
	}
}

void Qaplay::plCurrentLoad()					// load the current m3u
{
// cout << "plCurrentLoad: reached " << endl;                                               // *****  debug  *****
	liM3uItems = m3uItemGetList(sM3u);
	if (liM3uItems.count() == 1) {					// check if error or only empty
		strM3uItemD = liM3uItems.at(0);
		if (strM3uItemD.sSongName.left(6) == tr("Error!")) {
			QMessageBox::information(0, tr("qaplay error - could not read playlist!"), tr("%1").arg(strM3uItemD.sSongName));
			return;
		}
	}
	iM3uLoaded = 2;
//cout << "plCurrentLoad: liM3uItems.count() = " << QString::number(liM3uItems.count()).toStdString() << endl;                                               // *****  debug  *****
}

void Qaplay::plCurrentPlayList()				// play current playlist
{
	if (iM3uLoaded < 2) {
		plCurrentSetup();
	}
	
	if (iM3uLoaded == 2) {
		player->stop();
		sFileInvoked = sM3u;
		playReadDir();	
	} else {
		QMessageBox::information(0, tr("qaplay information - no favorites playlist!"), tr("There is no internal playlist, yet. Nothing to play."));
		return;
	}
}

void Qaplay::plCurrentSongAdd()					// add current song to current playlist
{
	if (iM3uLoaded != 2) {							// first call, need to check and load playlist if available - here when needed, not during start - speed path
		plCurrentSetup();
	}

	if (m3uItemExists(slSongs.at(iSongCurListIdx), liM3uItems) == false) {	// song (full path/name) not already in list - don't add it twice
//		slM3u.append("#EXTINF:" + QString::number(metaData.value(QMediaMetaData::Duration).toInt() / 1000) + "," + metaData.metaDataKeyToString(QMediaMetaData::AlbumArtist) + " - " + metaData.metaDataKeyToString(QMediaMetaData::Title));
//		slM3u.append(slSongs.at(iSongCurListIdx));
		strM3uItemD.sSec = QString::number(metaData.value(QMediaMetaData::Duration).toInt() / 1000);
		strM3uItemD.sSongName = metaData.metaDataKeyToString(QMediaMetaData::AlbumArtist) + " - " + metaData.metaDataKeyToString(QMediaMetaData::Title);
		strM3uItemD.sSongPathName = slSongs.at(iSongCurListIdx);
		liM3uItems.append(strM3uItemD);
		if (plCurrentWrite() == false) {				// error while writing m3u immediately - internal playlist will be set to nil
			return;
		}
		showMessage(tr("qaplay info"),tr("Current song added to playlist:\n%1").arg(sM3u),QSystemTrayIcon::Information,12000);	// yeah, we did it!
	} else {
		showMessage(tr("qaplay info"),tr("This song is already in the playlist!\nNot adding it again."),QSystemTrayIcon::Information,12000);	
	}
}

void Qaplay::plCurrentUpdated(const QList<StrM3u>& liM3uItemsD)	// update current playlist from edit playlist widget
{
// cout << "plCurrentLoad: reached " << endl;                                               // *****  debug  *****
	liM3uItems = liM3uItemsD;
	plCurrentWrite();														// internal m3u will be set to nil in case of error
//cout << "plCurrentLoad: liM3uItems.count() = " << QString::number(liM3uItems.count()).toStdString() << endl;                                               // *****  debug  *****
}

bool Qaplay::plCurrentWrite()					// save current favorites playlist
{
	if (m3uFileWrite(sM3u, liM3uItems) == 0) {				// error while writing m3u immediately
		showMessage(tr("qaplay error"),tr("Sorry, an error occurred while trying to write the playlist:\n%1").arg(sM3u),QSystemTrayIcon::Information,12000);
		iM3uLoaded = 0;
		sM3u = "";											// ******************* need to do better error handling here - but should never happen?
		settingsWrite();
		return false;
	} else {
		return true;
	}
}
/*
void Qaplay::plCurrentSongDel()			// delete selected song from playlist
{

}

void Qaplay::plCurrentSongRename()		// rename current song in playlist
{

}
*/


//  **********************************
//  *****  edit playlist widget  *****
//  **********************************
void Qaplay::editPlaylistWdgtDelete()			// delete edit playlist widget
{
	eplw->deleteLater();
	iWShown = 0;
}

void Qaplay::editPlaylistWdgtShow()				// show edit playlist widget
{
	if (iWShown == 0) {
		if (iM3uLoaded != 2) {						// load current playlist if it was not used before - speed path
			plCurrentSetup();
		}
		if (liM3uItems.count() > 0) {			// do not show empty playlist - struct list does not have first line #EXTM3U entry, 1 line per double line m3u entry
			eplw = new EditPlayList();
			connect(this, SIGNAL(sigQapEplShow(const QString&, const QList<StrM3u>&)), eplw, SLOT(eplDlgShow(const QString&, const QList<StrM3u>&)));
			connect(eplw, SIGNAL(sigEplQapM3uUpd(const QList<StrM3u>&)), this, SLOT(plCurrentUpdated(const QList<StrM3u>&)));
			connect(eplw, SIGNAL(sigEplQapDelete()), this, SLOT(editPlaylistWdgtDelete()));
			iWShown = 4;
			emit sigQapEplShow(sM3u, liM3uItems);	// m3u full path name + m3u struct list
		} else {
			QMessageBox::information(0, tr("qaplay - empty playlist!"),
										tr("The internal playlist to add songs to is empty.\n"
										   "No need to open the window."));
		}
	}
}


//  **********************************
//  *****  set/manage playlists  *****
//  **********************************
/*void Qaplay::setPlayListPlDelete()			// delete a playlist
{

}
*/

void Qaplay::setPlayListPlUpdated(const QString& sD)	// current m3u has been updated - set
{
	sM3u = sD;
	plCurrentLoad();
	settingsWrite();
}

void Qaplay::setPlayListPlNew(const QString& sD)	// create a new, empty playlist - not from the set playlist widget
{
	sM3u = sD;
	if (m3uFileCreate(sM3u) == "0") {
		QMessageBox::warning(0, tr("qaplay error - could not create playlist!"),
		                        tr("Failed to create the playlist\n"
								   "%1").arg(sM3u));
	} else {
		plCurrentLoad();
	}
}

void Qaplay::setPlayListPlRenamed()				// current playlist has been renamed
{

}

void Qaplay::setPlayListWdgtShow()				// show set/manage playlists widget
{
//cout << "setPlayListShow: reached " << endl;                                               // *****  debug  *****
	if (iWShown == 0) {
		splw = new SetPlayList();
		connect(this, SIGNAL(sigQapSplShow(const QString&)), splw, SLOT(splDlgShow(const QString&)));
		connect(splw, SIGNAL(sigSplQapM3uUpd(const QString&)), this, SLOT(setPlayListPlUpdated(const QString&)));
		connect(splw, SIGNAL(sigSplQapDelete()), this, SLOT(setPlayListWdgtDelete()));
		if (iM3uLoaded != 2) {					// load current playlist if it was not used before - speed path
		plCurrentSetup();
		}
		iWShown = 3;
		emit sigQapSplShow(sM3u);
	}
}

void Qaplay::setPlayListWdgtDelete()			// hide set/manage playlists widget
{
	splw->deleteLater();
	iWShown = 0;
}


//  ******************************************
//  *****  internal song list functions  *****
//  ******************************************
void Qaplay::songListWdgtShow()						// show songlist
{
//cout << "setPlayListShow: reached " << endl;                                               // *****  debug  *****
	if (iWShown == 0) {
		slw = new SongList();
		connect(this, SIGNAL(sigQapSlShow(int, const QString&, const QStringList&)), slw, SLOT(slDlgShow(int, const QString&, const QStringList&)));
		connect(slw, SIGNAL(sigSlQapSongChngd(int)), this, SLOT(songListWdgtSongChngd(int)));
		connect(slw, SIGNAL(sigSlQapDelete()), this, SLOT(songListWdgtDelete()));
/*		if (iM3uLoaded != 2) {					// load current playlist if it was not used before - speed path
		plCurrentSetup();
		}*/
		iWShown = 5;
		emit sigQapSlShow(iSongCurListIdx, sFileInvoked, slSongs);
	}
}

void Qaplay::songListWdgtDelete()					// delete songlist window
{
	slw->deleteLater();
	iWShown = 0;
}

void Qaplay::songListWdgtSongChngd(int iD)			// play another song from the list
{
	iSongCurListIdx = iD;
	playAt();
}


//  ************************************
//  *****  show about/help dialog  *****
//  ************************************
void Qaplay::ahAbout()							// show about dialog (1)
{
	if (iWShown == 0) {
		iWShown = 1;
		ahShow();
	}
}

void Qaplay::ahHelp()							// show help dialog (2)
{
	if (iWShown == 0) {
		iWShown = 2;
		ahShow();
	}
}

void Qaplay::ahShow()							// show about/help dialog
{
	ahw = new AboutHelp();
	connect(this, SIGNAL(sigQapAh(int)), ahw, SLOT(showAh(int)));
	connect(ahw, SIGNAL(sigAhQapDelete()), this, SLOT(ahDelete()));
	emit sigQapAh(iWShown);
}

void Qaplay::ahDelete()							// delete about/help dialog
{
	ahw->deleteLater();
	iWShown = 0;
}
